@@ -1,29 +1,17 @@
 use strict; use warnings;
 package Inline::Module;
-our $VERSION = '0.22';
+our $VERSION = '0.25';
+our $API_VERSION = 'v2';
 
 use Config();
 use File::Path();
 use File::Find();
 use Carp 'croak';
 
-# use XXX;
-
 my $inline_build_path = './blib/Inline';
 
-sub new {
-    my $class = shift;
-    return bless {@_}, $class;
-}
-
 use constant DEBUG_ON => $ENV{PERL_INLINE_MODULE_DEBUG} ? 1 : 0;
-sub DEBUG {
-    if (DEBUG_ON) {
-        print ">>>>>> ";
-        printf @_;
-        print "\n";
-    }
-}
+sub DEBUG { if (DEBUG_ON) { print "DEBUG >>> ", sprintf(@_), "\n" }}
 
 #------------------------------------------------------------------------------
 # This import serves multiple roles:
@@ -39,40 +27,39 @@ sub import {
     my ($stub_module, $program) = caller;
 
     if ($program eq 'Makefile.PL' && not -e 'INLINE.h') {
+        $class->check_inc_inc($program);
         no warnings 'once';
         *MY::postamble = \&postamble;
         return;
     }
+    elsif ($program eq 'Build.PL') {
+        $class->check_inc_inc($program);
+    }
 
     return unless @_;
     my $cmd = shift;
 
-    return $class->handle_makestub(@_)
-        if $cmd eq 'makestub';
+    return $class->handle_stub($stub_module, @_)
+        if $cmd eq 'stub';
     return $class->handle_autostub(@_)
         if $cmd eq 'autostub';
     return $class->handle_distdir(@ARGV)
         if $cmd eq 'distdir';
     return $class->handle_fixblib()
         if $cmd eq 'fixblib';
-
-    if ($cmd =~ /^v[0-9]$/) {
-        $class->check_api_version($stub_module, $cmd => @_);
-        no strict 'refs';
-        *{"${stub_module}::import"} = $class->importer($stub_module);
-        return;
-    }
+    return $class->handle_makestub(@_)
+        if $cmd eq 'makestub';
 
     die "Unknown Inline::Module::import argument '$cmd'"
 }
 
 sub check_api_version {
-    my ($class, $stub_module, $api_version, $inline_module_version) = @_;
-    if ($api_version ne 'v1' or $inline_module_version ne $VERSION) {
+    my ($class, $stub_module, $api_version) = @_;
+    if ($api_version ne $API_VERSION) {
         warn <<"...";
 It seems that '$stub_module' is out of date.
-It is using Inline::Module version '$inline_module_version'.
-You have Inline::Module version '$VERSION' installed.
+It is using Inline::Module API version '$api_version'.
+You have Inline::Module API version '$API_VERSION' installed.
 
 Make sure you have the latest version of Inline::Module installed, then run:
 
@@ -85,6 +72,21 @@ Make sure you have the latest version of Inline::Module installed, then run:
     }
 }
 
+sub check_inc_inc {
+    my ($class, $program) = @_;
+    my $first = $INC[0] or die;
+    if ($first !~ /^(\.[\/\\])?inc[\/\\]?$/) {
+        die <<"...";
+First element of \@INC should be 'inc'.
+It's '$first'.
+Add this line to the top of your '$program':
+
+    use lib 'inc';
+
+...
+    }
+}
+
 sub importer {
     my ($class, $stub_module) = @_;
     return sub {
@@ -108,28 +110,28 @@ sub importer {
 }
 
 #------------------------------------------------------------------------------
-# The postamble methods:
+# The postamble method:
 #------------------------------------------------------------------------------
 sub postamble {
     my ($makemaker, %args) = @_;
 
-    my $inline = $args{inline}
+    my $meta = $args{inline}
         or croak "'postamble' section requires 'inline' key in Makefile.PL";
     croak "postamble 'inline' section requires 'module' key in Makefile.PL"
-        unless $inline->{module};
+        unless $meta->{module};
 
-    my $self = $Inline::Module::Self = Inline::Module->new;
-    $self->default_args($inline, $makemaker);
+    my $class = __PACKAGE__;
+    $class->default_meta($meta);
 
-    my $code_modules = $self->{module};
-    my $stub_modules = $self->{stub};
-    my @included_modules = $self->included_modules;
+    my $code_modules = $meta->{module};
+    my $stub_modules = $meta->{stub};
+    my $included_modules = $class->included_modules($meta);
 
     my $section = <<"...";
 distdir : distdir_inline
 
 distdir_inline : create_distdir
-\t\$(NOECHO) \$(ABSPERLRUN) -MInline::Module=distdir -e 1 -- \$(DISTVNAME) @$stub_modules -- @included_modules
+\t\$(NOECHO) \$(ABSPERLRUN) -MInline::Module=distdir -e 1 -- \$(DISTVNAME) @$stub_modules -- @$included_modules
 
 pure_all ::
 ...
@@ -143,61 +145,30 @@ pure_all ::
     return $section;
 }
 
-sub default_args {
-    my ($self, $args, $makemaker) = @_;
-    $args->{module} = $makemaker->{NAME} unless $args->{module};
-    $args->{module} = [ $args->{module} ] unless ref $args->{module};
-    $args->{stub} ||= [ map "${_}::Inline", @{$args->{module}} ];
-    $args->{stub} = [ $args->{stub} ] unless ref $args->{stub};
-    $args->{ilsm} ||= 'Inline::C';
-    $args->{ilsm} = [ $args->{ilsm} ] unless ref $args->{ilsm};
-    %$self = %$args;
+#------------------------------------------------------------------------------
+# The handle methods.
+#------------------------------------------------------------------------------
+sub handle_stub {
+    my ($class, $stub_module, $api_version) = @_;
+    $class->check_api_version($stub_module, $api_version);
+    no strict 'refs';
+    *{"${stub_module}::import"} = $class->importer($stub_module);
+    return;
 }
 
-sub included_modules {
-    my ($self) = @_;
-    my $ilsm = $self->{ilsm};
-    my @include = (
-        'Inline',
-        'Inline::denter',
-        'Inline::Module',
-        @$ilsm,
-    );
-    if (grep /:C$/, @$ilsm) {
-        push @include,
-            'Inline::C::Parser::RegExp';
-    }
-    if (grep /:CPP$/, @$ilsm) {
-        push @include,
-            'Inline::CPP::Parser::RecDescent',
-            'Parse::RecDescent';
-    }
-    return @include;
-}
-
-#------------------------------------------------------------------------------
-# Class methods.
-#------------------------------------------------------------------------------
 sub handle_makestub {
     my ($class, @args) = @_;
 
-    my $dest;
     my @modules;
     for my $arg (@args) {
-        if ($arg eq 'blib') {
-            $dest = 'blib/lib';
-        }
-        elsif ($arg =~ m!/!) {
-            $dest = $arg;
-        }
-        elsif ($arg =~ /::/) {
+        if ($arg =~ /::/) {
             push @modules, $arg;
         }
         else {
             croak "Unknown 'makestub' argument: '$arg'";
         }
     }
-    $dest ||= 'lib';
+    my $dest = 'lib';
 
     for my $module (@modules) {
         my $code = $class->proxy_module($module);
@@ -223,79 +194,42 @@ sub handle_autostub {
     require lib;
     lib->import('lib');
 
-    my ($dest, %autostub_modules);
+    my %autostub_modules;
     for my $arg (@args) {
         if ($arg =~ m!::!) {
             $autostub_modules{$arg} = 1;
         }
-        elsif ($arg =~ m!(^(b?lib$|mem)$|/)!) {
-            croak "Invalid arg '$arg'. Dest path already set to '$dest'"
-                if $dest;
-            $dest = $arg eq 'blib' ? 'blib/lib' : $arg;
-        }
         else {
             croak "Unknown 'autostub' argument: '$arg'";
         }
     }
-    $dest ||= 'mem';
 
     push @INC, sub {
-        my ($self, $module_path) = @_;
+        my ($this, $module) = @_;
         delete $ENV{PERL5OPT};
 
-        my $module = $module_path;
         $module =~ s!\.pm$!!;
         $module =~ s!/!::!g;
         $autostub_modules{$module} or return;
 
-        if ($dest eq 'mem') {
-            my $code = $class->proxy_module($module);
-            open my $fh, '<', \$code;
-            return $fh;
-        }
-
-        my $path = "$dest/$module_path";
-        if (not -e $path) {
-            my $code = $class->proxy_module($module);
-            $class->write_module($dest, $module, $code);
-            print "Created stub module '$path' (Inline::Module $VERSION)\n";
-        }
-
-        open my $fh, '<', $path or die "Can't open '$path' for input:\n$!";
+        my $code = $class->proxy_module($module);
+        open my $fh, '<', \$code;
         return $fh;
     }
 }
 
 sub handle_distdir {
     my ($class, $distdir, @args) = @_;
-    my (@inlined_modules, @included_modules);
+    my $stub_modules = [];
+    my $included_modules = [];
 
     while (@args and ($_ = shift(@args)) ne '--') {
-        push @inlined_modules, $_;
+        push @$stub_modules, $_;
     }
     while (@args and ($_ = shift(@args)) ne '--') {
-        push @included_modules, $_;
+        push @$included_modules, $_;
     }
-
-    my @manifest; # files created under distdir
-    for my $module (@inlined_modules) {
-        my $code = $class->dyna_module($module);
-        $class->write_module("$distdir/lib", $module, $code);
-        $code = $class->proxy_module($module);
-        $class->write_module("$distdir/inc", $module, $code);
-        $module =~ s!::!/!g;
-        push @manifest, "lib/$module.pm", "inc/$module.pm";
-    }
-    for my $module (@included_modules) {
-        my $code = $class->read_local_module($module);
-        $class->write_module("$distdir/inc", $module, $code);
-        $module =~ s!::!/!g;
-        push @manifest, "inc/$module.pm";
-    }
-
-    $class->add_to_manifest($distdir, @manifest);
-
-    return @manifest; # return a list of the files added
+    $class->add_to_distdir($distdir, $stub_modules, $included_modules);
 }
 
 sub handle_fixblib {
@@ -319,6 +253,69 @@ sub handle_fixblib {
     }, $inline_build_path);
 }
 
+#------------------------------------------------------------------------------
+# Worker methods.
+#------------------------------------------------------------------------------
+sub default_meta {
+    my ($class, $meta) = @_;
+    defined $meta->{module}
+        or die "Meta 'module' not defined";
+    $meta->{module} = [ $meta->{module} ] unless ref $meta->{module};
+    $meta->{stub} ||= [ map "${_}::Inline", @{$meta->{module}} ];
+    $meta->{stub} = [ $meta->{stub} ] unless ref $meta->{stub};
+    $meta->{ilsm} ||= 'Inline::C';
+    $meta->{ilsm} = [ $meta->{ilsm} ] unless ref $meta->{ilsm};
+}
+
+sub included_modules {
+    my ($class, $meta) = @_;
+    my $ilsm = $meta->{ilsm};
+    my $include = [
+        'Inline',
+        'Inline::denter',
+        'Inline::Module',
+        @$ilsm,
+    ];
+    if (caller eq 'Module::Build::InlineModule') {
+        push @$include, 'Module::Build::InlineModule';
+    }
+    if (grep /:C$/, @$ilsm) {
+        push @$include,
+            'Inline::C::Parser::RegExp';
+    }
+    if (grep /:CPP$/, @$ilsm) {
+        push @$include,
+            'Inline::C',
+            'Inline::CPP::Config',
+            'Inline::CPP::Parser::RecDescent',
+            'Parse::RecDescent';
+    }
+    return $include;
+}
+
+sub add_to_distdir {
+    my ($class, $distdir, $stub_modules, $included_modules) = @_;
+    my $manifest = []; # files created under distdir
+    for my $module (@$stub_modules) {
+        my $code = $class->dyna_module($module);
+        $class->write_module("$distdir/lib", $module, $code);
+        $code = $class->proxy_module($module);
+        $class->write_module("$distdir/inc", $module, $code);
+        $module =~ s!::!/!g;
+        push @$manifest, "lib/$module.pm", "inc/$module.pm";
+    }
+    for my $module (@$included_modules) {
+        my $code = $class->read_local_module($module);
+        $class->write_module("$distdir/inc", $module, $code);
+        $module =~ s!::!/!g;
+        push @$manifest, "inc/$module.pm";
+    }
+
+    $class->add_to_manifest($distdir, @$manifest);
+
+    return $manifest; # return a list of the files added
+}
+
 sub read_local_module {
     my ($class, $module) = @_;
     eval "require $module; 1" or die $@;
@@ -343,11 +340,14 @@ sub proxy_module {
 # This module is for author-side development only. When this module is shipped
 # to CPAN, it will be automagically replaced with content that does not
 # require any Inline framework modules (or any other non-core modules).
+#
+# To regenerate this stub module, run this command:
+#
+#   perl -MInline::Module=makestub,$module
 
 use strict; use warnings;
 package $module;
-use Inline::Module 'v1' => '$VERSION';
-
+use Inline::Module stub => '$API_VERSION';
 1;
 ...
 }
@@ -394,12 +394,15 @@ sub write_module {
 sub add_to_manifest {
     my ($class, $distdir, @files) = @_;
     my $manifest = "$distdir/MANIFEST";
-    open my $out, '>>', $manifest
-        or die "Can't open '$manifest' for append:\n$!";
-    for my $file (@files) {
-        print $out "$file\n";
+
+    if (-w $manifest) {
+        open my $out, '>>', $manifest
+            or die "Can't open '$manifest' for append:\n$!";
+        for my $file (@files) {
+            print $out "$file\n";
+        }
+        close $out;
     }
-    close $out;
 }
 
 1;
